import javax.swing.JTextField;
import javax.swing.JLabel;
import javax.swing.BoxLayout;
import javax.swing.JPanel;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import java.awt.FlowLayout;
import javax.swing.JFrame;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.Date;
import javax.swing.WindowConstants;

/**
 * Class that lets a user guess a number and tells them if they are right.
 * 
 * @author Barbara Ericson ericson@cc.gatech.edu
 */
public class NumberGuesser extends JPanel implements ActionListener
{
  ////////////////////////// Fields /////////////////////////////////
  
  /** Picks a number and tells you if your guess is right */
  private NumberPicker numberPicker = new NumberPicker();
  
  /** Number of tries */
  private int numTries = 0;
  
  /** Current guess */
  private int guess = 0;
  
  /** Textfield to let user enter guess */
  private JTextField guessTextField = new JTextField(5);
  
  /** Label to ask uesr to guess */
  private JLabel guessLabel = new JLabel("Enter a number between 1 and 10 ");
  
  /** Label to show answer */
  private JLabel answerLabel = new JLabel("Then click the \"Submit\" button.");
  
  /** Label to show the number of tries */
  private JLabel numTriesLabel = new JLabel(" ");
  
  /** Label to check the guess */
  private JButton checkGuessButton = new JButton("Submit");
  
  /////////////////////// Constructors //////////////////////////////
  
  /**
   * Constructor that takes no arguments
   */
  public NumberGuesser()
  {
    initPanel();
  }
  
 
  //////////////////////// Methods ////////////////////////////////////
  
  /**
   * Method to handle the button push
   */
  public void actionPerformed(ActionEvent e)
  {
    // get the guess from the text field
    String guessString = guessTextField.getText();
    
    // increment the number of tries
    numTries++;
    numTriesLabel.setText("You made " + numTries + " guesses.");
    
    // get the guess as an int
    try {
      
      // get the guess as an int
      guess = Integer.parseInt(guessString);
      
      // check if it is right
      if (numberPicker.isCorrect(guess))
      {
        answerLabel.setText("That is right!");
        numberPicker.pickANumber();
        numTries = 0;
      }
      else
      {
        answerLabel.setText("No, it isn't " + guessString + ".");
      }
    } catch (NumberFormatException ex) {
      answerLabel.setText("Please try a number!");
    }
    
  }
  
  /** 
   * Method to set-up the panel for display
   */
  private void initPanel()
  {
    // set the layout to a flowlayout
    this.setLayout(new FlowLayout());
    
    // create a vertical boxx
    Box box = Box.createVerticalBox();
    
    // add the guess label
    box.add(guessLabel);
    
    // add the text field
    box.add(guessTextField);
    
    // add the answer label
    box.add(answerLabel);
    
    // add the number of tries
    box.add(numTriesLabel);
    
    // add the checkGuess button and when it is pushed notify this panel
    box.add(checkGuessButton);
    checkGuessButton.addActionListener(this);
    
    // add the box to this panel
    this.add(box);
  }
  
  /** 
   * Main to test
   */
  public static void main(String[] argv)
  {
    
    // create a main window
    JFrame frame = new JFrame("Number Guess");
    
    // create a number guesser panel and add it to the main window
    frame.getContentPane().add(new NumberGuesser());
    
    // tell the frame to get rid of the window when the x is clicked on
    //frame.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
    
    // make the frame whatever size it needs to be to fit the contents
    frame.pack();
    
    // show the frame
    frame.setVisible(true);
  }

}