/*
 * PersonInputFrame.java
 * 
 * Copyright (c) 2004 ICEGT, Georgia Institute of Technology, Jeremy Goecks
 */
package edu.gatech.icegt.guesswho;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.ButtonModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.border.Border;

/**
 * The frame that provides an interface for entering a person into the system or
 * updating a person in the system.
 * 
 * @author Jeremy Goecks
 * @version 0.1
 */
public class PersonInputFrame extends JFrame 
	implements GuessWhoGameConstants, ActionListener {
	
	/** Image width. */
	protected static final int IMAGE_WIDTH = 128;
	
	/** 'No picture' file. */
	protected File noPicFile;
	
	/** Directory where person files are to be stored. */
	protected String personFilesDir;
	
	/** Directory where person image files are found. */
	protected String personImagesDir;
	
	/** Label that shows person image's. */
	protected JLabel personImageLabel;
	
	/** Label that shows file location for person's image. */
	protected JLabel personImageFileLocLabel;
	
	/** Name text field. */
	protected JTextField nameTextField;
	
	/** Hair color combo box. */
	protected JComboBox hairColorCb;
	
	/** Eye color combo box. */
	protected JComboBox eyeColorCb;
	
	/** Earrings buttons group. */
	protected ButtonGroup earringsButtonGroup;
	
	/** Gender buttons group. */
	protected ButtonGroup genderButtonGroup;
	
	/** Fun fact text area. */
	protected JTextArea funFactTa;
	
	/** Create frame. */
	public PersonInputFrame(String noPicFile, String personFilesDir, 
							String personImagesDir) {
		//
		// Init.
		//
		this.noPicFile = new File(noPicFile);
		this.personFilesDir = personFilesDir;
		this.personImagesDir = personImagesDir;
		
		//
		// Add personal image frame to NORTH panel.
		//
		getContentPane().add(createPersonImagePanel(), BorderLayout.NORTH);
		
		//
		// Add personal information labels and inputs to center panel.
		//
		getContentPane().add(createPersonalInformationPanel(), BorderLayout.CENTER);
		
		//
		// Set frame attributes.
		//
		setTitle("Guess Who: Add/Update Person");
	} // end PersonInputFrame()
	
	/**
	 * Create personal information panel
	 * @return
	 */
	protected JPanel createPersonalInformationPanel() {
		// 
		// Create panel and set layout.
		//
		JPanel personalInfoPanel = new JPanel();
		GridBagLayout gbl = new GridBagLayout();
		personalInfoPanel.setLayout(gbl);
		GridBagConstraints gbc = new GridBagConstraints();
		
		//
		// Add border to panel.
		//
		Border etchedBorder = BorderFactory.createEtchedBorder();
		Border titledBorder = 
			BorderFactory.createTitledBorder(etchedBorder, "Personal Information");
		personalInfoPanel.setBorder(titledBorder);
		
		//
		// Add name label and text field to panel.
		//
		
		// Create label, text field.
		JLabel label = new JLabel("name:");
		nameTextField = new JTextField(15);
		
		// Add label, text field to a temp panel and add panel to top panel.
		gbc.gridx = GridBagConstraints.REMAINDER;
		gbc.gridy = 0;
		JPanel tempPanel = new JPanel();
		gbl.setConstraints(tempPanel, gbc);
		tempPanel.add(label);
		tempPanel.add(nameTextField);
		personalInfoPanel.add(tempPanel);	
		
		//
		// Add gender label and radio buttons.
		//
		
		// Create label, button group, and buttons.
		label = new JLabel("gender?");
		genderButtonGroup = new ButtonGroup();
		JRadioButton maleButton = new JRadioButton("male");
		maleButton.setActionCommand("male");
		genderButtonGroup.add(maleButton);
		JRadioButton femaleButton = new JRadioButton("female");
		femaleButton.setActionCommand("female");
		genderButtonGroup.add(femaleButton);
		
		// Put radio buttons on panel and add to input controls panel.
		JPanel radioButttonsPanel = new JPanel();
		radioButttonsPanel.add(maleButton);
		radioButttonsPanel.add(femaleButton);
		
		// Add label, buttons panel.
		gbc.gridx = GridBagConstraints.REMAINDER;
		gbc.gridy = 1;
		tempPanel = new JPanel();
		gbl.setConstraints(tempPanel, gbc);
		tempPanel.add(label);
		tempPanel.add(radioButttonsPanel);
		personalInfoPanel.add(tempPanel);
		
		//
		// Add hair color label and combo box to panel.
		//
		label = new JLabel("hair color:");
		hairColorCb = new JComboBox(Person.VALID_HAIR_COLORS);
		
		// Add label, combo box.
		gbc.gridx = GridBagConstraints.REMAINDER;
		gbc.gridy = 2;
		tempPanel = new JPanel();
		gbl.setConstraints(tempPanel, gbc);
		tempPanel.add(label);
		tempPanel.add(hairColorCb);
		personalInfoPanel.add(tempPanel);
		
		//
		// Add eye color label and combo box to panel.
		//
		label = new JLabel("eye color:");
		eyeColorCb = new JComboBox(Person.VALID_EYE_COLORS);
		
		// Add label, combo box.
		gbc.gridx = GridBagConstraints.REMAINDER;
		gbc.gridy = 3;
		tempPanel = new JPanel();
		gbl.setConstraints(tempPanel, gbc);
		tempPanel.add(label);
		tempPanel.add(eyeColorCb);
		personalInfoPanel.add(tempPanel);
		
		//
		// Add earrings label and radio buttons.
		//
		
		// Create label, button group, and buttons.
		label = new JLabel("earrings?");
		earringsButtonGroup = new ButtonGroup();
		JRadioButton yesButton = new JRadioButton("yes");
		yesButton.setActionCommand("yes");
		earringsButtonGroup.add(yesButton);
		JRadioButton noButton = new JRadioButton("no");
		noButton.setActionCommand("no");
		earringsButtonGroup.add(noButton);
		
		// Put radio buttons on panel and add to input controls panel.
		radioButttonsPanel = new JPanel();
		radioButttonsPanel.add(yesButton);
		radioButttonsPanel.add(noButton);
		
		// Add label, buttons panel.
		gbc.gridx = GridBagConstraints.REMAINDER;
		gbc.gridy = 4;
		tempPanel = new JPanel();
		gbl.setConstraints(tempPanel, gbc);
		tempPanel.add(label);
		tempPanel.add(radioButttonsPanel);
		personalInfoPanel.add(tempPanel);
		
		//
		// Add fun fact text area to panel.
		//
		label = new JLabel("fun fact:");
		funFactTa = new JTextArea(3, 15);
		
		// Put text area in scroll pane.
		JScrollPane taScrollPane = 
			new JScrollPane(funFactTa, 
							JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, 
							JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		
		// Add label, scroll pane.
		gbc.gridx = GridBagConstraints.REMAINDER;
		gbc.gridy = 5;
		tempPanel = new JPanel();
		gbl.setConstraints(tempPanel, gbc);
		tempPanel.add(label);
		tempPanel.add(taScrollPane);
		personalInfoPanel.add(tempPanel);
		
		//
		// Add submit button to panel.
		//
		JButton submitButton = new JButton("Add person");
		gbc.gridx = GridBagConstraints.REMAINDER;
		gbc.gridy = 6;
		gbc.gridwidth = 2;
		gbl.setConstraints(submitButton, gbc);
		personalInfoPanel.add(submitButton);
		
		// Add frame as button's action listener.
		submitButton.addActionListener(this);
		
		return personalInfoPanel;
	} // end createPersonalInformationPanel()
	

	/**
	 * Creates a panel that manages a person's image.
	 */
	protected JPanel createPersonImagePanel() {
		//
		// Create panel and set layout.
		//
		JPanel personImagePanel = new JPanel();
		GridBagLayout gbl = new GridBagLayout();
		personImagePanel.setLayout(gbl);
		GridBagConstraints gbc = new GridBagConstraints();
		
		//
		// Add titled border to panel.
		//
		Border etchedBorder = BorderFactory.createEtchedBorder();
		Border titledBorder = 
			BorderFactory.createTitledBorder(etchedBorder, "Personal Image");
		personImagePanel.setBorder(titledBorder);
		
		//
		// Add ImageIcon to panel; initial image is NO_IMAGE.
		//
		
		// Create icon.
		ImageIcon personImage = 
			GuessWhoGame.getImageIconWithScaledImage(noPicFile, IMAGE_WIDTH, this);
		personImageLabel = new JLabel(personImage);
		
		// Add to panel.
		gbc.gridx = GridBagConstraints.REMAINDER;
		gbc.gridy = 0;
		gbc.gridwidth = 2;
		gbl.setConstraints(personImageLabel, gbc);
		personImagePanel.add(personImageLabel);
		
		//
		// Add photo file label.
		//
		
		// Create label.
		personImageFileLocLabel = new JLabel("no image");
		
		// Add label.
		gbc.gridx = 0;
		gbc.gridy = 1;
		gbc.gridwidth = 1;
		gbl.setConstraints(personImageFileLocLabel, gbc);
		personImagePanel.add(personImageFileLocLabel);
		
		//
		// Add button that opens a file chooser and updates the image to the file
		// that the user chooses. Image must reside in the personImagesDir
		//
		JButton openFileChooserBtn = new JButton("Change image");
		openFileChooserBtn.addActionListener(
				new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						// Open dialog so that user can choose file.
						File startDir = new File(personImagesDir);
						JFileChooser chooser = new JFileChooser(startDir);
						int returnVal = chooser.showOpenDialog(PersonInputFrame.this);
						
						// Verify choice and, if valid, update photo.
					    if (returnVal == JFileChooser.APPROVE_OPTION) {
					    	// Get selected file.
					        File file = chooser.getSelectedFile();
					        
					        // If file does not reside in the personImagesDir,
					        // show error.
					        if (!file.getParentFile().equals(startDir.getAbsoluteFile()))
					        	JOptionPane.showMessageDialog(PersonInputFrame.this, 
					        		"Image files must reside in the directory '" + personImagesDir + "'",
					        		"Image file in not in the necessary location",
									JOptionPane.ERROR_MESSAGE);
					        else
					        	// Update image.
					        	updatePersonImage(file);
					    } else {
					    	// Open cancelled by user.
					    } // end else
						
					} // end actionPerformed()
				} // end ActionListener()
											);
		
		// Add button to panel.
		gbc.gridx = 1;
		gbl.setConstraints(openFileChooserBtn, gbc);
		personImagePanel.add(openFileChooserBtn);
		
		return personImagePanel;
	} // end createPersonImagePanel()
	
	
	/**
	 * Handles ActionEvent that is triggered by submit button. Method trys to 
	 * create a Person object from the data that user entered and write the
	 * person to a file 
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		//
		// Get data to create Person object. If there are errors, show dialog
		// with error message and exit.
		//
		boolean noErrors = true;
		String errorMsg = "";
		
		// Get name.
		String name = nameTextField.getText();
		// Is name non-empty?
		if (name.equals("")) {
			// Error.
			noErrors = false;
			errorMsg = "You must specify the person's name";
		} // end if
		
		// Get photo file.
		String photoFile = personImageFileLocLabel.getText();
		// Is name non-empty?
		if (photoFile.equals("")) {
			// Error.
			noErrors = false;
			errorMsg = errorMsg + (errorMsg != "" ? "<br>" : "") + "Invalid photoFile: " + photoFile;
		} // end if
		
		// Get fun fact file.
		String funFact = funFactTa.getText();
		// Is name non-empty?
		if (funFact.equals("")) {
			// Error.
			noErrors = false;
			errorMsg = errorMsg + (errorMsg != "" ? "<br>" : "") + "You must specify a fun fact";
		} // end if
		
		// Get hair color, eye color. There are no possible errors for these
		// elements.
		String 
			hairColor = (String)hairColorCb.getSelectedItem(),
			eyeColor = (String)eyeColorCb.getSelectedItem();
		
		// Get hasEarrings from radio buttons.
		ButtonModel hasEarringsBM = earringsButtonGroup.getSelection();
		boolean hasEarrings = false;
		if (hasEarringsBM == null) {
			// Error.
			noErrors = false;
			errorMsg = errorMsg + (errorMsg != "" ? "<br>" : "") + "You must indicate whether the person has earrings";
		} // end if
		else
			hasEarrings = (hasEarringsBM.getActionCommand().equals("yes"));
		
		// Get gender from radio buttons.
		ButtonModel genderBM = genderButtonGroup.getSelection();
		String gender = "";
		if (genderBM == null) {
			// Error.
			noErrors = false;
			errorMsg = errorMsg + (errorMsg != "" ? "<br>" : "") + "You must indicate the person's gender";
		} // end if
		else 
			gender = genderBM.getActionCommand();
		
		//
		// If there are no errors, create person and write to file. Otherwise,
		// output error message.
		//
		if (noErrors) {
			// Create person.
			Person person = 
				new Person(name, photoFile, hairColor, eyeColor, 
								gender, hasEarrings, funFact);
			
			// Get file name to write person to; ues the person's name (without 
			// spaces) as the file name.
			StringBuffer fileName = new StringBuffer(name);
			int index = -1;
			while ( (index = fileName.indexOf(" ")) != -1)
				fileName.deleteCharAt(index);
		
			// Write person to file.
			String fullFilePath = 
				personFilesDir + File.separator + fileName.toString() + 
				PERSON_FILE_EXT;
			File outFile = new File(fullFilePath);
			boolean success = false;
			try { success = person.writeToFile(outFile); }
			catch (IOException ioe) {
				ioe.printStackTrace();
			}
			
			// Output alert to indicate whether write was successful.
			String alertMsg = 
				(success ? "Wrote " + name + "'s data to " + outFile : 
							"ERROR: could not write to " + outFile);
			JOptionPane.showMessageDialog(this, alertMsg, "Success!", 
											JOptionPane.INFORMATION_MESSAGE);
			
			// If successful, clear fields.
			if (success) {
				updatePersonImage(noPicFile);
				nameTextField.setText("");
				//hairColorCb.
				//eyeColorCb.
				//earringsButtonGroup.setSelected()
				//genderButtonGroup.setSelected()
				funFactTa.setText("");
			} // end if
		} // end if
		else {
			// Error.
			JOptionPane.showMessageDialog(this, "<html>" + errorMsg + "</html>",
											"Failed", JOptionPane.ERROR_MESSAGE);
		} // end else
	} // end actionPerformed()
	
	/**
	 * Updates person's image in the frame.
	 */
	protected void updatePersonImage(File newImageFile) {
		//
		// Update person image label, image file label.
		//
		personImageLabel.setIcon(
				GuessWhoGame.getImageIconWithScaledImage(newImageFile, IMAGE_WIDTH, this)
								);
		personImageFileLocLabel.setText(newImageFile.getName());
	} // end updatePersonImage()
	
	
} // end class PersonInputFrame

